@@ -43,16 +43,21 @@ impl Client {
                     .unwrap()
             })
             .clone();
-        let conn = existing.get().await.map_err(|err| match err {
-            PoolError::Timeout(_) => panic!("should never see deadpool timeout"),
-            PoolError::Closed => panic!("closed"),
-            PoolError::NoRuntimeSpecified => panic!("what"),
-            PoolError::Backend(err) => err,
-            PoolError::PostCreateHook(_) => todo!(),
-            PoolError::PreRecycleHook(_) => todo!(),
-            PoolError::PostRecycleHook(_) => todo!(),
-        })?;
-        Ok(conn)
+        loop {
+            let conn = existing.get().await.map_err(|err| match err {
+                PoolError::Timeout(_) => panic!("should never see deadpool timeout"),
+                PoolError::Closed => panic!("closed"),
+                PoolError::NoRuntimeSpecified => panic!("what"),
+                PoolError::Backend(err) => err,
+                PoolError::PostCreateHook(_) => todo!(),
+                PoolError::PreRecycleHook(_) => todo!(),
+                PoolError::PostRecycleHook(_) => todo!(),
+            })?;
+            if conn.expired() {
+                continue;
+            }
+            return Ok(conn);
+        }
     }
 
     /// Does a melnet request to any given endpoint.
@@ -92,48 +97,47 @@ impl Client {
         // let _guard = GLOBAL_LIMIT.acquire().await;
         let start = Instant::now();
         // grab a connection
-        loop {
-            let mut conn = self.connect(addr).await.map_err(MelnetError::Network)?;
-            if conn.expired() {
-                continue;
-            }
-            let res = async {
-                // send a request
-                let rr = stdcode::serialize(&RawRequest {
-                    proto_ver: PROTO_VER,
-                    netname: netname.to_owned(),
-                    verb: verb.to_owned(),
-                    payload: stdcode::serialize(&req).unwrap(),
-                })
-                .unwrap();
-                write_len_bts(conn.as_mut().as_mut(), &rr).await?;
-                // read the response length
-                let response: RawResponse = stdcode::deserialize(
-                    &read_len_bts(conn.as_mut().as_mut()).await?,
-                )
-                .map_err(|e| {
-                    MelnetError::Network(std::io::Error::new(std::io::ErrorKind::InvalidData, e))
-                })?;
-                let response = match response.kind.as_ref() {
-                    "Ok" => stdcode::deserialize::<TOutput>(&response.body)
-                        .map_err(|_| MelnetError::Custom("stdcode error".to_owned()))?,
-                    "NoVerb" => return Err(MelnetError::VerbNotFound),
-                    _ => {
-                        return Err(MelnetError::Custom(
-                            String::from_utf8_lossy(&response.body).to_string(),
-                        ))
-                    }
-                };
-                let elapsed = start.elapsed();
-                if elapsed.as_secs_f64() > 3.0 {
-                    log::warn!("melnet req to {} took {:?}", addr, elapsed)
+        let mut conn = self.connect(addr).await.map_err(MelnetError::Network)?;
+
+        let res = async {
+            // send a request
+            let rr = stdcode::serialize(&RawRequest {
+                proto_ver: PROTO_VER,
+                netname: netname.to_owned(),
+                verb: verb.to_owned(),
+                payload: stdcode::serialize(&req).unwrap(),
+            })
+            .unwrap();
+            write_len_bts(conn.as_mut().as_mut(), &rr).await?;
+            // read the response length
+            let response: RawResponse = stdcode::deserialize(
+                &read_len_bts(conn.as_mut().as_mut()).await?,
+            )
+            .map_err(|e| {
+                MelnetError::Network(std::io::Error::new(std::io::ErrorKind::InvalidData, e))
+            })?;
+            let response = match response.kind.as_ref() {
+                "Ok" => stdcode::deserialize::<TOutput>(&response.body)
+                    .map_err(|_| MelnetError::Custom("stdcode error".to_owned()))?,
+                "NoVerb" => return Err(MelnetError::VerbNotFound),
+                _ => {
+                    return Err(MelnetError::Custom(
+                        String::from_utf8_lossy(&response.body).to_string(),
+                    ))
                 }
-                Ok::<_, crate::MelnetError>(response)
-            };
-            return match res.await {
-                Err(err) => Err(err),
-                x => x,
             };
+            let elapsed = start.elapsed();
+            if elapsed.as_secs_f64() > 3.0 {
+                log::warn!("melnet req to {} took {:?}", addr, elapsed)
+            }
+            Ok::<_, crate::MelnetError>(response)
+        };
+        match res.await {
+            Err(err) => {
+                let _ = Object::take(conn);
+                Err(err)
+            }
+            x => x,
         }
     }
 }
